package java.net;

import java.io.IOException;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.io.InputStream;
import sun.net.idn.Punycode;
import java.text.ParseException;
import sun.text.normalizer.UCharacterIterator;
import sun.net.idn.StringPrep;

public final class IDN
{
    public static final int ALLOW_UNASSIGNED = 1;
    public static final int USE_STD3_ASCII_RULES = 2;
    private static final String ACE_PREFIX = "xn--";
    private static final int ACE_PREFIX_LENGTH;
    private static final int MAX_LABEL_LENGTH = 63;
    private static StringPrep namePrep;
    
    public static String toASCII(String s, int n) {
        int i = 0;
        StringBuffer sb = new StringBuffer();
        if (isRootLabel(s)) {
            return ".";
        }
        while (i < s.length()) {
            int searchDots = searchDots(s, i);
            sb.append(toASCIIInternal(s.substring(i, searchDots), n));
            if (searchDots != s.length()) {
                sb.append('.');
            }
            i = searchDots + 1;
        }
        return sb.toString();
    }
    
    public static String toASCII(String s) {
        return toASCII(s, 0);
    }
    
    public static String toUnicode(String s, int n) {
        int i = 0;
        StringBuffer sb = new StringBuffer();
        if (isRootLabel(s)) {
            return ".";
        }
        while (i < s.length()) {
            int searchDots = searchDots(s, i);
            sb.append(toUnicodeInternal(s.substring(i, searchDots), n));
            if (searchDots != s.length()) {
                sb.append('.');
            }
            i = searchDots + 1;
        }
        return sb.toString();
    }
    
    public static String toUnicode(String s) {
        return toUnicode(s, 0);
    }
    
    private IDN() {
    }
    
    private static String toASCIIInternal(String str, int n) {
        int allASCII = isAllASCII(str);
        StringBuffer sb;
        if (allASCII == 0) {
            UCharacterIterator instance = UCharacterIterator.getInstance(str);
            try {
                sb = IDN.namePrep.prepare(instance, n);
            }
            catch (ParseException cause) {
                throw new IllegalArgumentException(cause);
            }
        }
        else {
            sb = new StringBuffer(str);
        }
        if (sb.length() == 0) {
            throw new IllegalArgumentException("Empty label is not a legal name");
        }
        if ((n & 0x2) != 0x0) {
            int i = 0;
            while (i < sb.length()) {
                if (isNonLDHAsciiCodePoint((int)sb.charAt(i))) {
                    throw new IllegalArgumentException("Contains non-LDH ASCII characters");
                }
                ++i;
            }
            if (sb.charAt(0) == '-' || sb.charAt(sb.length() - 1) == '-') {
                throw new IllegalArgumentException("Has leading or trailing hyphen");
            }
        }
        if (allASCII == 0 && !isAllASCII(sb.toString())) {
            if (!startsWithACEPrefix(sb)) {
                StringBuffer encode;
                try {
                    encode = Punycode.encode(sb, null);
                }
                catch (ParseException cause2) {
                    throw new IllegalArgumentException(cause2);
                }
                sb = toASCIILower(encode);
                sb.insert(0, "xn--");
            }
            else {
                throw new IllegalArgumentException("The input starts with the ACE Prefix");
            }
        }
        if (sb.length() > 63) {
            throw new IllegalArgumentException("The label in the input is too long");
        }
        return sb.toString();
    }
    
    private static String toUnicodeInternal(String str, int n) {
        StringBuffer prepare;
        if (!isAllASCII(str)) {
            try {
                prepare = IDN.namePrep.prepare(UCharacterIterator.getInstance(str), n);
            }
            catch (Exception ex) {
                return str;
            }
        }
        else {
            prepare = new StringBuffer(str);
        }
        if (startsWithACEPrefix(prepare)) {
            String substring = prepare.substring(IDN.ACE_PREFIX_LENGTH, prepare.length());
            try {
                StringBuffer decode = Punycode.decode(new StringBuffer(substring), null);
                if (toASCII(decode.toString(), n).equalsIgnoreCase(prepare.toString())) {
                    return decode.toString();
                }
            }
            catch (Exception ex2) {}
        }
        return str;
    }
    
    private static boolean isNonLDHAsciiCodePoint(int n) {
        return (0 <= n && n <= 44) || (46 <= n && n <= 47) || (58 <= n && n <= 64) || (91 <= n && n <= 96) || (123 <= n && n <= 127);
    }
    
    private static int searchDots(String s, int index) {
        for (; index < s.length() && !isLabelSeparator(s.charAt(index)); ++index) {}
        return index;
    }
    
    private static boolean isRootLabel(String s) {
        return s.length() == 1 && isLabelSeparator(s.charAt(0));
    }
    
    private static boolean isLabelSeparator(char c) {
        return c == '.' || c == '。' || c == '．' || c == '｡';
    }
    
    private static boolean isAllASCII(String s) {
        boolean n = true;
        int i = 0;
        while (i < s.length()) {
            if (s.charAt(i) > '\u007f') {
                n = false;
                break;
            }
            ++i;
        }
        return n;
    }
    
    private static boolean startsWithACEPrefix(StringBuffer sb) {
        boolean n = true;
        if (sb.length() < IDN.ACE_PREFIX_LENGTH) {
            return false;
        }
        for (int i = 0; i < IDN.ACE_PREFIX_LENGTH; ++i) {
            if (toASCIILower(sb.charAt(i)) != "xn--".charAt(i)) {
                n = false;
            }
        }
        return n;
    }
    
    private static char toASCIILower(char c) {
        if ('A' <= c && c <= 'Z') {
            return (char)(c + 97 - 65);
        }
        return c;
    }
    
    private static StringBuffer toASCIILower(StringBuffer sb) {
        StringBuffer sb2 = new StringBuffer();
        for (int i = 0; i < sb.length(); ++i) {
            sb2.append(toASCIILower(sb.charAt(i)));
        }
        return sb2;
    }
    
    static {
        ACE_PREFIX_LENGTH = "xn--".length();
        IDN.namePrep = null;
        try {
            InputStream resourceAsStream;
            if (System.getSecurityManager() != null) {
                resourceAsStream = (InputStream)AccessController.doPrivileged(new PrivilegedAction<InputStream>() {
                    @Override
                    public InputStream run() {
                        return StringPrep.class.getResourceAsStream("uidna.spp");
                    }
                });
            }
            else {
                resourceAsStream = StringPrep.class.getResourceAsStream("uidna.spp");
            }
            IDN.namePrep = new StringPrep(resourceAsStream);
            resourceAsStream.close();
        }
        catch (IOException ex) {
            assert false;
        }
    }
}
